/* -*- C++ -*- Time-stamp: <10 Mar 01 18:09:02 Michael Bischoff> */

/*****************************************************************************/
/*									     */
/*									     */
/*	QMatchIt version 1.0 -- file fields.h				     */
/*									     */
/*	Include file for the superclasses of the field classes of QMatchIt   */
/*	written by Michael Bischoff (Michael.Bischoff@gmx.net)		     */
/*	November-2000							     */
/*	see COPYRIGHT.qmatchit for Copyright details			     */
/*									     */
/*									     */
/*****************************************************************************/

#ifndef FIELDS_H
#define FIELDS_H

#include "field.h"			// defines the base class
#include "fieldGraphics.h"		// defines the base class

#define RTTI_CURSOR	'*'		// not in level file
#define RTTI_BOX	'#'		// '1'..'8' in level file, subtype 1..8
#define RTTI_SAVESPOT	'S'		// save spot
#define RTTI_TELEPORT	'T'		// teleporter (red/green/blue) subtype 0..2
#define RTTI_UNSAFE	'U'		// vanishes when block on it (grey)
#define RTTI_VANISHER	'V'		// vanishes when block away (blue)
#define RTTI_WALL	'W'		// solid wall
#define RTTI_REMOVABLE	'X'		// removable wall

#define RTTI_BOMB	'B'		// bomb!
#define RTTI_INVERTER	'I'		// inverts magnets
#define RTTI_MAGNET	'M'		// magnets (subtype 0..3)
#define RTTI_WEIGHT	'G'		// gravity weight
#define RTTI_ONEWAY	'O'		// '<', '>' in level-file (subtype 0, 1)
#define RTTI_BALLOON	'^'		// balloon moves up if field above is free
#define RTTI_LIFT	'!'		// lift moves up 1 box, _ is other lift, & is up and down

class f_cursor : public field {		// no subtype
    friend class field;
protected:
    int rtti()		const { return RTTI_CURSOR; }
    double getZ()	const { return 31.0; }	// on top of everything
    int baseFrame()	const { return 24; }
    int currentFrame();
    bool isAnimated()	const { return true; }
};

class f_unsafe : public field {
    friend class field;
protected:
    int rtti()		const { return RTTI_UNSAFE; }
    int baseFrame()	const { return 1; }
    int currentFrame();
};
  
class f_vanisher : public f_unsafe {
    friend class field;
protected:
    int rtti()		const { return RTTI_VANISHER; }
    int baseFrame()	const { return 9; }
};
  
class f_teleporter : public field {	  // subtypes 0, 1, 2 (colors)
    friend class field;
    static const char asciitab[3];
protected:
    int rtti()		const { return RTTI_TELEPORT; }
    int baseFrame()	const { return 27 + subtype; }
    int ascii()		const { return static_cast<int>(asciitab[subtype]); }
};
  
class f_wall : public field {
    friend class field;
protected:
    int rtti()		const { return RTTI_WALL; }
    int baseFrame()	const { return 0; }
};

class f_savespot: public field {
    friend class field;
protected:
    int rtti()		const { return RTTI_SAVESPOT; }
    int baseFrame()	const { return 33; }
};

// f_flickers: any object which flickers when vanishing
// pure virtual, derived by f_box, f_bomb, f_inverter, f_removable_wall
class f_flickers : public field {
    friend class field;
protected:
    int currentFrame();
};

class f_bomb : public f_flickers {
    friend class field;
protected:
    int rtti()		const { return RTTI_BOMB; }
    int baseFrame()	const { return 32; }
};

class f_removable : public f_flickers {
    friend class field;
protected:
    int rtti()		const { return RTTI_REMOVABLE; }
    int baseFrame()	const { return 34; }
    bool isSelectable() const { return true; }
};

class f_box : public f_flickers {	// subtype 0 (weight) and 1..8
    friend class field;
protected:
    int rtti()		const { return RTTI_BOX; }
    int baseFrame()	const { return subtype ? 16 - 1 + subtype : 43; }
    bool feelsGravity()	const { return true; }
    bool feelsMagnets()	const { return true; }
    bool isSelectable() const { return true; }
    int ascii()		const { return '0' + subtype; }	// box!
};

class f_inverter : public f_flickers {
    friend class field;
protected:
    int rtti()		const { return RTTI_INVERTER; }
    int baseFrame()	const { return 35; }
    bool isSelectable() const { return true; }
};

class f_magnet : public field {
    friend class field;
    static const char asciitab[4];
public:
    static const int magnet_left  = 0;	// subtype value
    static const int magnet_up    = 1;	// subtype value
    static const int magnet_right = 2;	// subtype value
    static const int magnet_down  = 3;	// subtype value
    void invert()	{ subtype ^= 2; gr->setAnimated(true); }
protected:
    int rtti()		const { return RTTI_MAGNET; }
    int baseFrame()	const { return 36 + subtype; }
    int ascii()		const { return static_cast<int>(asciitab[subtype]); }
};

class f_oneway : public field {
    friend class field;
public:
    static const int oneway_left  = 0;	// subtype value
    static const int oneway_right = 1;	// subtype value
protected:
    int rtti()		const { return RTTI_ONEWAY; }
    int baseFrame()	const { return 40 + subtype; }
    int ascii()		const { return subtype == oneway_left ? '<' : '>'; }
};

class f_weight : public field {
    friend class field;
protected:
    int rtti()		const { return RTTI_WEIGHT; }
    int baseFrame()	const { return 42; }
    bool feelsGravity()	const { return true; }
};

class f_balloon : public field {
    friend class field;
protected:
    int rtti()		const { return RTTI_BALLOON; }
    int baseFrame()	const { return 44; }
};

// lift moves up with no box, if subtype = 1
class f_lift : public field {
    friend class field;
    static const char asciitab[3];
protected:
    int rtti()		const { return RTTI_LIFT; }
    int baseFrame()	const { return 45 + subtype; }
    int ascii()		const { return static_cast<int>(asciitab[subtype]); }
};

#endif
